/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2009 John Fitzgibbons and others
Copyright (C) 2007-2008 Kristian Duske

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// draw.c -- 2d drawing

#include "quakedef.h"

//extern unsigned char d_15to8table[65536]; // johnfitz -- never used

cvar_t		scr_conalpha = {"scr_conalpha", "0.5", true}; // johnfitz

qpic_t		*draw_disc;
qpic_t		*draw_backtile;

gltexture_t *char_texture; // johnfitz
qpic_t		*pic_ovr, *pic_ins; // johnfitz -- new cursor handling
qpic_t		*pic_nul; // johnfitz -- for missing gfx, don't crash

// johnfitz -- new pics
byte pic_ovr_data[8][8] =
{
	{255, 255, 255, 255, 255, 255, 255, 255},
	{255, 15, 15, 15, 15, 15, 15, 255},
	{255, 15, 15, 15, 15, 15, 15,  2},
	{255, 15, 15, 15, 15, 15, 15,  2},
	{255, 15, 15, 15, 15, 15, 15,  2},
	{255, 15, 15, 15, 15, 15, 15,  2},
	{255, 15, 15, 15, 15, 15, 15,  2},
	{255, 255,  2,  2,  2,  2,  2,  2},
};

byte pic_ins_data[9][8] =
{
	{ 15, 15, 255, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{ 15, 15,  2, 255, 255, 255, 255, 255},
	{255,  2,  2, 255, 255, 255, 255, 255},
};

byte pic_nul_data[8][8] =
{
	{252, 252, 252, 252,  0,  0,  0,  0},
	{252, 252, 252, 252,  0,  0,  0,  0},
	{252, 252, 252, 252,  0,  0,  0,  0},
	{252, 252, 252, 252,  0,  0,  0,  0},
	{  0,  0,  0,  0, 252, 252, 252, 252},
	{  0,  0,  0,  0, 252, 252, 252, 252},
	{  0,  0,  0,  0, 252, 252, 252, 252},
	{  0,  0,  0,  0, 252, 252, 252, 252},
};

byte pic_stipple_data[8][8] =
{
	{255,  0,  0,  0, 255,  0,  0,  0},
	{  0,  0, 255,  0,  0,  0, 255,  0},
	{255,  0,  0,  0, 255,  0,  0,  0},
	{  0,  0, 255,  0,  0,  0, 255,  0},
	{255,  0,  0,  0, 255,  0,  0,  0},
	{  0,  0, 255,  0,  0,  0, 255,  0},
	{255,  0,  0,  0, 255,  0,  0,  0},
	{  0,  0, 255,  0,  0,  0, 255,  0},
};

byte pic_crosshair_data[8][8] =
{
	{255, 255, 255, 255, 255, 255, 255, 255},
	{255, 255, 255,  8,  9, 255, 255, 255},
	{255, 255, 255,  6,  8,  2, 255, 255},
	{255,  6,  8,  8,  6,  8,  8, 255},
	{255, 255,  2,  8,  8,  2,  2,  2},
	{255, 255, 255,  7,  8,  2, 255, 255},
	{255, 255, 255, 255,  2,  2, 255, 255},
	{255, 255, 255, 255, 255, 255, 255, 255},
};
// johnfitz

typedef struct
{
	gltexture_t *gltexture;
	float		sl, tl, sh, th;
} glpic_t;

ofs2dtype currentofs2d = OFS2D_NONE; // johnfitz -- for GL_Set2DOffsets

//==============================================================================
//
//  PIC CACHING
//
//==============================================================================

typedef struct cachepic_s
{
	char		name[MAX_QPATH];
	qpic_t		pic;
	byte		padding[32];	// for appended glpic
} cachepic_t;

#define	MAX_CACHED_PICS		128
cachepic_t	menu_cachepics[MAX_CACHED_PICS];
int			menu_numcachepics;

byte		menuplyr_pixels[4096];

//  scrap allocation
//  Allocate all the little status bar obejcts into a single texture
//  to crutch up stupid hardware / drivers

// mh - renamed to avoid the name #define in 2 different places with 2 different purposes
#define	MAX_SCRAPS		2
#define	SCRAP_BLOCK_WIDTH	256
#define	SCRAP_BLOCK_HEIGHT	256

int			scrap_allocated[MAX_SCRAPS][SCRAP_BLOCK_WIDTH];
byte		scrap_texels[MAX_SCRAPS][SCRAP_BLOCK_WIDTH * SCRAP_BLOCK_HEIGHT]; // johnfitz -- removed *4 after SCRAP_BLOCK_HEIGHT
qboolean	scrap_dirty;
gltexture_t	*scrap_textures[MAX_SCRAPS]; // johnfitz


/*
================
Scrap_AllocBlock

returns an index into scrap_texnums[] and the position inside it
================
*/
int Scrap_AllocBlock (int w, int h, int *x, int *y)
{
	int		i, j;
	int		best, best2;
	int		texnum;

	for (texnum = 0; texnum < MAX_SCRAPS; texnum++)
	{
		best = SCRAP_BLOCK_HEIGHT;

		for (i = 0; i < SCRAP_BLOCK_WIDTH - w; i++)
		{
			best2 = 0;

			for (j = 0; j < w; j++)
			{
				if (scrap_allocated[texnum][i+j] >= best)
					break;

				if (scrap_allocated[texnum][i+j] > best2)
					best2 = scrap_allocated[texnum][i+j];
			}

			if (j == w)
			{
				// this is a valid spot
				*x = i;
				*y = best = best2;
			}
		}

		if (best + h > SCRAP_BLOCK_HEIGHT)
			continue;

		for (i = 0; i < w; i++)
			scrap_allocated[texnum][*x + i] = best + h;

		return texnum;
	}

	Sys_Error ("Scrap_AllocBlock: full"); // johnfitz -- correct function name
	return 0; // johnfitz -- shut up compiler
}

/*
================
Scrap_Upload -- johnfitz -- now uses TexMgr
================
*/
void Scrap_Upload (void)
{
	char name[8];
	int	i;

	for (i = 0; i < MAX_SCRAPS; i++)
	{
		sprintf (name, "scrap%i", i);

		scrap_textures[i] = TM_LoadImage (NULL, name, SCRAP_BLOCK_WIDTH, SCRAP_BLOCK_HEIGHT, SRC_INDEXED, scrap_texels[i],
											  "", (src_offset_t) scrap_texels[i], TEXPREF_ALPHA | TEXPREF_OVERWRITE | TEXPREF_NOPICMIP);

		GL_BindTexture (GL_TEXTURE0, scrap_textures[i]);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	}

	scrap_dirty = false;
}

/*
================
Draw_PicFromWad
================
*/
qpic_t *Draw_PicFromWad (char *name)
{
	qpic_t	*p;
	glpic_t	gl;
	src_offset_t offset; // johnfitz

	p = (qpic_t *) W_GetLumpName (name);

	if (!p) return pic_nul; // johnfitz

	// load little ones into the scrap
	if (p->width < 64 && p->height < 64)
	{
		int		x, y;
		int		i, j, k;
		int		texnum;

		texnum = Scrap_AllocBlock (p->width, p->height, &x, &y);
		scrap_dirty = true;
		k = 0;

		for (i = 0; i < p->height; i++)
		{
			for (j = 0; j < p->width; j++, k++)
				scrap_texels[texnum][(y+i) *SCRAP_BLOCK_WIDTH + x + j] = p->data[k];
		}

		gl.gltexture = scrap_textures[texnum]; // johnfitz -- changed to an array
		// johnfitz -- no longer go from 0.01 to 0.99
		gl.sl = x / (float) SCRAP_BLOCK_WIDTH;
		gl.sh = (x + p->width) / (float) SCRAP_BLOCK_WIDTH;
		gl.tl = y / (float) SCRAP_BLOCK_HEIGHT;
		gl.th = (y + p->height) / (float) SCRAP_BLOCK_HEIGHT;
	}
	else
	{
		char texturename[64]; // johnfitz
		sprintf (texturename, "%s:%s", WADFILENAME, name); // johnfitz

		offset = (src_offset_t) p - (src_offset_t) wad_base + sizeof (int) * 2; // johnfitz

		gl.gltexture = TM_LoadImage (NULL, texturename, p->width, p->height, SRC_INDEXED, p->data, WADFILENAME,
										 offset, TEXPREF_ALPHA | TEXPREF_PAD | TEXPREF_NOPICMIP); // johnfitz -- TexMgr
		gl.sl = 0;
		gl.sh = (float) p->width / (float) TM_PadConditional (p->width); // johnfitz
		gl.tl = 0;
		gl.th = (float) p->height / (float) TM_PadConditional (p->height); // johnfitz
	}

	memcpy (p->data, &gl, sizeof (glpic_t));

	return p;
}

void Draw_InvalidateState (void);

/*
================
Draw_CachePic
================
*/
qpic_t	*Draw_CachePic (char *path)
{
	cachepic_t	*pic;
	int			i;
	qpic_t		*dat;
	glpic_t		gl;

	Draw_InvalidateState ();

	for (pic = menu_cachepics, i = 0; i < menu_numcachepics; pic++, i++)
	{
		if (!strcmp (path, pic->name))
			return &pic->pic;
	}

	if (menu_numcachepics == MAX_CACHED_PICS)
		Sys_Error ("menu_numcachepics == MAX_CACHED_PICS");

	menu_numcachepics++;
	strcpy (pic->name, path);

	// load the pic from disk
	dat = (qpic_t *) COM_LoadTempFile (path);

	if (!dat)
		Sys_Error ("Draw_CachePic: failed to load %s", path);

	SwapPic (dat);

	// HACK HACK HACK --- we need to keep the bytes for
	// the translatable player picture just for the menu
	// configuration dialog
	if (!strcmp (path, "gfx/menuplyr.lmp"))
		memcpy (menuplyr_pixels, dat->data, dat->width * dat->height);

	pic->pic.width = dat->width;
	pic->pic.height = dat->height;

	gl.gltexture = TM_LoadImage (NULL, path, dat->width, dat->height, SRC_INDEXED, dat->data, path,
									 sizeof (int) * 2, TEXPREF_ALPHA | TEXPREF_PAD | TEXPREF_NOPICMIP); // johnfitz -- TexMgr
	gl.sl = 0;
	gl.sh = (float) dat->width / (float) TM_PadConditional (dat->width); // johnfitz
	gl.tl = 0;
	gl.th = (float) dat->height / (float) TM_PadConditional (dat->height); // johnfitz
	memcpy (pic->pic.data, &gl, sizeof (glpic_t));

	return &pic->pic;
}


/*
================
Draw_MakePic -- johnfitz -- generate pics from internal data
================
*/
qpic_t *Draw_MakePic (char *name, int width, int height, byte *data)
{
	int flags = TEXPREF_NEAREST | TEXPREF_ALPHA | TEXPREF_PERSIST | TEXPREF_NOPICMIP | TEXPREF_PAD;
	qpic_t		*pic;
	glpic_t		gl;

	pic = (qpic_t *) Hunk_Alloc (sizeof (qpic_t) - 4 + sizeof (glpic_t));
	pic->width = width;
	pic->height = height;

	gl.gltexture = TM_LoadImage (NULL, name, width, height, SRC_INDEXED, data, "", (src_offset_t) data, flags);
	gl.sl = 0;
	gl.sh = (float) width / (float) TM_PadConditional (width);
	gl.tl = 0;
	gl.th = (float) height / (float) TM_PadConditional (height);
	memcpy (pic->data, &gl, sizeof (glpic_t));

	return pic;
}

//==============================================================================
//
//  INIT
//
//==============================================================================

/*
===============
Draw_LoadPics -- johnfitz
===============
*/
void Draw_LoadPics (void)
{
	byte		*data;
	src_offset_t	offset;

	data = (byte *) W_GetLumpName ("conchars");

	if (!data) Sys_Error ("Draw_LoadPics: couldn't load conchars");

	offset = (src_offset_t) data - (src_offset_t) wad_base;
	char_texture = TM_LoadImage (NULL, WADFILENAME":conchars", 128, 128, SRC_INDEXED, data,
									 WADFILENAME, offset, TEXPREF_ALPHA | TEXPREF_NEAREST | TEXPREF_NOPICMIP | TEXPREF_CONCHARS);

	draw_disc = Draw_PicFromWad ("disc");
	draw_backtile = Draw_PicFromWad ("backtile");

	// precache all of our menu pics so that they're valid when we come to use them, otherwise we get weirdness
	// when they're cached on-demand with the new renderer code.
	Draw_CachePic ("gfx/bigbox.lmp");
	Draw_CachePic ("gfx/box_bl.lmp");
	Draw_CachePic ("gfx/box_bm.lmp");
	Draw_CachePic ("gfx/box_br.lmp");
	Draw_CachePic ("gfx/box_ml.lmp");
	Draw_CachePic ("gfx/box_mm.lmp");
	Draw_CachePic ("gfx/box_mm2.lmp");
	Draw_CachePic ("gfx/box_mr.lmp");
	Draw_CachePic ("gfx/box_tl.lmp");
	Draw_CachePic ("gfx/box_tm.lmp");
	Draw_CachePic ("gfx/box_tr.lmp");
	Draw_CachePic ("gfx/complete.lmp");
	Draw_CachePic ("gfx/conback.lmp");
	Draw_CachePic ("gfx/dim_drct.lmp");
	Draw_CachePic ("gfx/dim_ipx.lmp");
	Draw_CachePic ("gfx/dim_modm.lmp");
	Draw_CachePic ("gfx/dim_mult.lmp");
	Draw_CachePic ("gfx/dim_tcp.lmp");
	Draw_CachePic ("gfx/finale.lmp");
	Draw_CachePic ("gfx/help0.lmp");
	Draw_CachePic ("gfx/help1.lmp");
	Draw_CachePic ("gfx/help2.lmp");
	Draw_CachePic ("gfx/help3.lmp");
	Draw_CachePic ("gfx/help4.lmp");
	Draw_CachePic ("gfx/help5.lmp");
	Draw_CachePic ("gfx/inter.lmp");
	Draw_CachePic ("gfx/loading.lmp");
	Draw_CachePic ("gfx/mainmenu.lmp");
	Draw_CachePic ("gfx/menudot1.lmp");
	Draw_CachePic ("gfx/menudot2.lmp");
	Draw_CachePic ("gfx/menudot3.lmp");
	Draw_CachePic ("gfx/menudot4.lmp");
	Draw_CachePic ("gfx/menudot5.lmp");
	Draw_CachePic ("gfx/menudot6.lmp");
	Draw_CachePic ("gfx/menuplyr.lmp");
	Draw_CachePic ("gfx/mp_menu.lmp");
	Draw_CachePic ("gfx/netmen1.lmp");
	Draw_CachePic ("gfx/netmen2.lmp");
	Draw_CachePic ("gfx/netmen3.lmp");
	Draw_CachePic ("gfx/netmen4.lmp");
	Draw_CachePic ("gfx/netmen5.lmp");
	Draw_CachePic ("gfx/pause.lmp");
	Draw_CachePic ("gfx/p_load.lmp");
	Draw_CachePic ("gfx/p_multi.lmp");
	Draw_CachePic ("gfx/p_option.lmp");
	Draw_CachePic ("gfx/p_save.lmp");
	Draw_CachePic ("gfx/qplaque.lmp");
	Draw_CachePic ("gfx/ranking.lmp");
	Draw_CachePic ("gfx/sp_menu.lmp");
	Draw_CachePic ("gfx/ttl_cstm.lmp");
	Draw_CachePic ("gfx/ttl_main.lmp");
	Draw_CachePic ("gfx/ttl_sgl.lmp");
	Draw_CachePic ("gfx/vidmodes.lmp");
}


/*
===============
Draw_NewGame -- johnfitz
===============
*/
void Draw_NewGame (void)
{
	cachepic_t	*pic;
	int			i;

	// empty scrap and reallocate gltextures
	memset (&scrap_allocated, 0, sizeof (scrap_allocated));
	memset (&scrap_texels, 255, sizeof (scrap_texels));
	Scrap_Upload (); //creates 2 empty gltextures

	// empty lmp cache
	for (pic = menu_cachepics, i = 0; i < menu_numcachepics; pic++, i++)
		pic->name[0] = 0;

	menu_numcachepics = 0;

	// reload wad pics
	W_LoadWadFile (); // johnfitz -- filename is now hard-coded for honesty
	Draw_LoadPics ();
	SCR_LoadPics ();
	Sbar_LoadPics ();
}

/*
===============
Draw_Init -- johnfitz -- rewritten
===============
*/
void Draw_Init (void)
{
	Cvar_RegisterVariable (&scr_conalpha, NULL);

	// clear scrap and allocate gltextures
	memset (&scrap_allocated, 0, sizeof (scrap_allocated));
	memset (&scrap_texels, 255, sizeof (scrap_texels));
	Scrap_Upload (); //creates 2 empty textures

	// create internal pics
	pic_ins = Draw_MakePic ("ins", 8, 9, &pic_ins_data[0][0]);
	pic_ovr = Draw_MakePic ("ovr", 8, 8, &pic_ovr_data[0][0]);
	pic_nul = Draw_MakePic ("nul", 8, 8, &pic_nul_data[0][0]);

	// load game pics
	Draw_LoadPics ();
}


//==============================================================================
//
//  2D DRAWING
//
//==============================================================================


float xofs_2d = 0;
float yofs_2d = 0;
float xscale_2d = 1;
float yscale_2d = 1;

// note - this just needs to be set to an invalid pointer here; it can't be NULL because NULL is a valid test
gltexture_t *draw_lasttexture = (gltexture_t *) 0xffffffff;
r_defaultquad_t *r_draw_quads = NULL;

void Draw_EndBatching (void)
{
	if (r_num_quads)
		GL_DrawPrimitive (GL_QUADS, 0, r_num_quads * 4);

	r_draw_quads = r_default_quads;
	r_num_quads = 0;
}


void Draw_Vertex (r_defaultquad_t *rdq, float x, float y, unsigned int *color, float s, float t)
{
	rdq->xyz[0] = x;
	rdq->xyz[1] = y;
	rdq->xyz[2] = 0;

	rdq->rgba = color[0];

	rdq->st[0] = s;
	rdq->st[1] = t;
}


void Draw_Textured2DQuad (float x, float y, float w, float h, float sl, float tl, float sh, float th, byte *color)
{
	r_defaultquad_t *dst = NULL;	// this might change below so we can't set it here
	byte defaultcolor[4] = {255, 255, 255, 255};
	byte *realcolor = color ? color : defaultcolor;

	x *= xscale_2d;
	y *= yscale_2d;
	x += xofs_2d;
	y += yofs_2d;
	w *= xscale_2d;
	h *= yscale_2d;

	if (r_num_quads + 4 >= r_max_quads)
		Draw_EndBatching ();

	// now we can safely set the dst pointer
	dst = (r_defaultquad_t *) r_draw_quads;
	r_draw_quads += 4;
	r_num_quads++;

	Draw_Vertex (&dst[0], x, y, (unsigned int *) realcolor, sl, tl);
	Draw_Vertex (&dst[1], x + w, y, (unsigned int *) realcolor, sh, tl);
	Draw_Vertex (&dst[2], x + w, y + h, (unsigned int *) realcolor, sh, th);
	Draw_Vertex (&dst[3], x, y + h, (unsigned int *) realcolor, sl, th);
}


void Draw_Colored2DQuad (float x, float y, float w, float h, byte *color)
{
	Draw_Textured2DQuad (x, y, w, h, 0, 0, 0, 0, color);
}


void Draw_InvalidateState (void)
{
	Draw_EndBatching ();

	// note again - not NULL because NULL is a valid state
	draw_lasttexture = (gltexture_t *) 0xffffffff;
}


void Draw_DisableTexture (void)
{
	Draw_EndBatching ();
	GL_TexEnv (GL_TEXTURE0, GL_TEXTURE_2D, GL_NONE);
}


void Draw_EnableTexture (void)
{
	Draw_EndBatching ();
	GL_TexEnv (GL_TEXTURE0, GL_TEXTURE_2D, GL_MODULATE);
}


void Draw_TestState (gltexture_t *texture)
{
	if (texture != draw_lasttexture)
	{
		Draw_EndBatching ();

		GL_SetStreamSource (0, GLSTREAM_POSITION, 3, GL_FLOAT, sizeof (r_defaultquad_t), r_default_quads->xyz);
		GL_SetStreamSource (0, GLSTREAM_COLOR, 4, GL_UNSIGNED_BYTE, sizeof (r_defaultquad_t), r_default_quads->color);
		GL_SetStreamSource (0, GLSTREAM_TEXCOORD1, 0, GL_NONE, 0, NULL);
		GL_SetStreamSource (0, GLSTREAM_TEXCOORD2, 0, GL_NONE, 0, NULL);

		if (texture)
		{
			GL_SetStreamSource (0, GLSTREAM_TEXCOORD0, 2, GL_FLOAT, sizeof (r_defaultquad_t), r_default_quads->st);
			GL_BindTexture (GL_TEXTURE0, texture);
		}
		else GL_SetStreamSource (0, GLSTREAM_TEXCOORD0, 0, GL_NONE, 0, NULL);

		draw_lasttexture = texture;
	}
}


/*
================
Draw_CharacterQuad -- johnfitz -- seperate function to spit out verts
================
*/
void Draw_CharacterQuad (int x, int y, char num, byte *color)
{
	int				row, col;
	float			frow, fcol, size;

	row = num >> 4;
	col = num & 15;

	frow = row * 0.0625;
	fcol = col * 0.0625;
	size = 0.0625;

	Draw_Textured2DQuad (x, y, 8, 8, fcol, frow, fcol + size, frow + size, color);
}


/*
================
Draw_Character -- johnfitz -- modified to call Draw_CharacterQuad
================
*/
void Draw_Character (int x, int y, int num)
{
	unsigned int rgba = 0xffffffff;

	if (y <= -8)
		return;			// totally off screen

	num &= 255;

	if (num == 32)
		return; //don't waste verts on spaces

	Draw_TestState (char_texture);
	Draw_CharacterQuad (x, y, (char) num, (byte *) &rgba);
}


void Draw_ColoredCharacter (int x, int y, int num, unsigned int color)
{
	if (y <= -8)
		return;			// totally off screen

	num &= 255;

	if (num == 32)
		return; //don't waste verts on spaces

	Draw_TestState (char_texture);
	Draw_CharacterQuad (x, y, (char) num, (byte *) &color);
}


/*
================
Draw_String -- johnfitz -- modified to call Draw_CharacterQuad
================
*/
void Draw_String (int x, int y, char *str)
{
	unsigned int rgba = 0xffffffff;

	if (y <= -8)
		return;			// totally off screen

	Draw_TestState (char_texture);

	while (*str)
	{
		if (*str != 32) //don't waste verts on spaces
			Draw_CharacterQuad (x, y, *str, (byte *) &rgba);

		str++;
		x += 8;
	}
}

/*
=============
Draw_Pic -- johnfitz -- modified
=============
*/
void Draw_ColoredPic (int x, int y, qpic_t *pic, byte *color)
{
	glpic_t			*gl;

	if (scrap_dirty)
	{
		Draw_EndBatching ();
		Scrap_Upload ();
		scrap_dirty = false;
	}

	gl = (glpic_t *) pic->data;

	Draw_TestState (gl->gltexture);
	Draw_Textured2DQuad (x, y, pic->width, pic->height, gl->sl, gl->tl, gl->sh, gl->th, color);
}


void Draw_Pic (int x, int y, qpic_t *pic)
{
	Draw_ColoredPic (x, y, pic, NULL);
}


/*
=============
Draw_TransPicTranslate -- johnfitz -- rewritten to use texmgr to do translation

Only used for the player color selection menu
=============
*/
void Draw_TransPicTranslate (int x, int y, qpic_t *pic, int top, int bottom)
{
	static int oldtop = -2;
	static int oldbottom = -2;
	gltexture_t *glt;
	glpic_t *glp;

	if (top != oldtop || bottom != oldbottom)
	{
		oldtop = top;
		oldbottom = bottom;
		glp = (glpic_t *) pic->data;
		glt = glp->gltexture;
		TM_ReloadImage (glt, top, bottom);
	}

	Draw_EndBatching ();
	Draw_Pic (x, y, pic);
}


/*
================
Draw_ConsoleBackground -- johnfitz -- rewritten
================
*/
void Draw_ConsoleBackground (void)
{
	qpic_t *pic;
	float alpha;

	pic = Draw_CachePic ("gfx/conback.lmp");
	pic->width = vid.conwidth;
	pic->height = vid.conheight;

	alpha = (con_forcedup) ? 1.0 : scr_conalpha.value;

	//	GL_Set2DOffsets (OFS2D_CONSOLE); //in case this is called from weird places

	if (alpha > 0.0)
	{
		byte color[4] = {255, 255, 255, 255};
		color[3] = BYTE_CLAMPF (alpha);

		Draw_ColoredPic (0, 0, pic, color);
	}
}


/*
=============
Draw_TileClear

This repeats a 64*64 tile graphic to fill the screen around a sized down
refresh window.
=============
*/
void Draw_TileClear (int x, int y, int w, int h)
{
	glpic_t	*gl;

	gl = (glpic_t *) draw_backtile->data;

	Draw_TestState (gl->gltexture);
	Draw_Textured2DQuad (x, y, w, h, x / 64.0, y / 64.0, (x + w) / 64.0, (y + h) / 64.0, NULL);
}


/*
=============
Draw_Fill

Fills a box of pixels with a single color
=============
*/
void Draw_Fill (int x, int y, int w, int h, int c, float alpha) // johnfitz -- added alpha
{
	byte *fillcolor = (byte *) &d_8to24table_rgba[c];
	fillcolor[3] = BYTE_CLAMPF (alpha);

	Draw_TestState (NULL);
	Draw_DisableTexture ();
	Draw_Colored2DQuad (x, y, w, h, fillcolor);
	Draw_EnableTexture ();
}


/*
================
Draw_FadeScreen -- johnfitz -- revised
================
*/
void Draw_FadeScreen (void)
{
	byte fadecolor[4] = {0, 0, 0, 160};

	GL_Set2DOffsets (OFS2D_DEFAULT);

	Draw_TestState (NULL);
	Draw_DisableTexture ();
	Draw_Colored2DQuad (0, 0, glwidth, glheight, fadecolor);
	Draw_EnableTexture ();
}


/*
================
Draw_BeginDisc

Draws the little blue disc in the corner of the screen.
Call before beginning any disc IO.
================
*/
void Draw_BeginDisc (void)
{
	// mh - direct writing to the front buffer is not actually supported on most modern gfx chips!
}


/*
================
GL_Set2DOffsets

================
*/
void GL_Set2DOffsets (ofs2dtype newofs2d)
{
	float s;
	int lines;

	if (newofs2d == currentofs2d)
		return;

	currentofs2d = newofs2d;

	// enforce defaults
	xofs_2d = 0;
	yofs_2d = 0;
	xscale_2d = 1;
	yscale_2d = 1;

	switch (newofs2d)
	{
	case OFS2D_CONSOLE:
		lines = vid.conheight - ((scr_con_current * vid.conheight) / glheight);

		xofs_2d = 0;
		yofs_2d = scr_con_current - glheight;

		xscale_2d = (float) glwidth / (float) vid.conwidth;
		yscale_2d = (float) glheight / (float) vid.conheight;
		break;

	case OFS2D_MENU:
		s = min ((float) glwidth / 320.0, (float) glheight / 200.0);
		s = CLAMP (1.0, scr_menuscale.value, s);

		xofs_2d = (glwidth - (320 * s)) / 2;
		yofs_2d = (glheight - (200 * s)) / 2;

		xscale_2d = s;
		yscale_2d = s;
		break;

	case OFS2D_SBAR:
		s = CLAMP (1.0, scr_sbarscale.value, (float) glwidth / 320.0);

		if (cl.gametype == GAME_DEATHMATCH)
			xofs_2d = 0;
		else xofs_2d = (glwidth - (320.0f * s)) / 2;

		switch (sb_lines)
		{
		case 24:
			yofs_2d = glheight - 48;
			break;

		default:
			yofs_2d = glheight - sb_lines;
			break;
		}

		xscale_2d = s;
		yscale_2d = s;

		break;

	case OFS2D_BOTTOMLEFT: // used by devstats
		s = (float) glwidth / (float) vid.conwidth; // use console scale

		xofs_2d = 0;
		yofs_2d = glheight - (200.0f * s);

		xscale_2d = s;
		yscale_2d = s;

		break;

	case OFS2D_BOTTOMRIGHT: // used by fps/clock
		s = (float) glwidth / (float) vid.conwidth; // use console scale

		xofs_2d = glwidth - (320.0f * s);
		yofs_2d = glheight - (200.0f * s);

		xscale_2d = s;
		yscale_2d = s;

		break;

	case OFS2D_DEFAULT:
	default:
		// just use default values from above
		break;
	}
}


/*
================
GL_Set2D -- johnfitz -- rewritten
================
*/
// mh - underwater warp!!!
void R_DrawUnderwaterWarp (void);

void GL_Set2D (void)
{
	currentofs2d = -1;

	Draw_InvalidateState ();

	// unbind everything and go back to TMU0
	GL_SetIndices (0, NULL);
	GL_SetStreamSource (0, GLSTREAM_POSITION, 0, GL_NONE, 0, NULL);
	GL_SetStreamSource (0, GLSTREAM_COLOR, 0, GL_NONE, 0, NULL);
	GL_SetStreamSource (0, GLSTREAM_TEXCOORD0, 0, GL_NONE, 0, NULL);
	GL_SetStreamSource (0, GLSTREAM_TEXCOORD1, 0, GL_NONE, 0, NULL);
	GL_SetStreamSource (0, GLSTREAM_TEXCOORD2, 0, GL_NONE, 0, NULL);

	// unbind all textures
	GL_BindTexture (GL_TEXTURE0, NULL);
	GL_BindTexture (GL_TEXTURE1, NULL);
	GL_BindTexture (GL_TEXTURE2, NULL);

	// unbined cached environments
	GL_TexEnv (GL_TEXTURE0, GL_TEXTURE_2D, GL_NONE);
	GL_TexEnv (GL_TEXTURE1, GL_TEXTURE_2D, GL_NONE);
	GL_TexEnv (GL_TEXTURE2, GL_TEXTURE_2D, GL_NONE);

	// back to TMU0
	GL_ActiveTexture (GL_TEXTURE0);

	GL_Set2DOffsets (OFS2D_DEFAULT);

	// partition the depth range for 3D/2D stuff
	glDepthRange (QGL_DEPTH_2D_BEGIN, QGL_DEPTH_2D_END);

	glViewport (0, 0, glwidth, glheight);

	glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
	glOrtho (0, glwidth, glheight, 0, -99999, 99999);

	glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();

	glDisable (GL_DEPTH_TEST);
	glDepthMask (GL_FALSE);
	glDisable (GL_CULL_FACE);

	// the underwater warp needs to be drawn before enabling alpha test or it won't blend properly
	R_DrawUnderwaterWarp ();

	glEnable (GL_BLEND);
	glDisable (GL_ALPHA_TEST);
	GL_TexEnv (GL_TEXTURE0, GL_TEXTURE_2D, GL_MODULATE);
}

